using System;
using System.Drawing.Printing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text;
using System.IO;

namespace GeoXplore
{
	/// <summary>
	/// Summary description for RbflnFunctions.
	/// </summary>
	
	
	
	public class RbflnFunctions
	{
		//----------------------------------------------------------------
		//----------------------------------------------------------------
		//  ex[N][Q] => Inputs of Q N-dimensional exemplar feature vectors
		//  w[N][M] => centers of Radias Basis Functions in hidden layers
		//  u[M][J] => weights from M hidden to J output nodes
		//  v[N][J] => weights from N input to J output nodes
		//  y[M][Q] => outputs from M hidden nodes for each of Q exemplars
		//  b[J]    => biases at the J output nodes
		//  t[J][Q] => target outputs from J output nodes, Qth exemplar vector
		//  z[J][Q] => computed outputs from J output nodes, Qth exemplar vector
		//  xmaxstd[N], xminstd[N]; => Max and min values for inputs  (standardized)
		//  tmaxstd[J],  tminstd[J];  => Max and min values for outputs (standardized)
		//  N <= 120, M <= 300; J <= 10; Q <= 2000
		//----------------------------------------------------------------
		public static int			N;										//number features/components in each feature vector
		public static int			M;										//number initial clusters to start clustering
		public static int			J;										//number components in label vectors
		public static int			Q;										//number of feature vectors in data file
		public static int           R;
		//public static int         K = 1;
		public static int           M1;
		public static int           Mref;
		public static int			stdFlag;								//standardization flag (0 or 1)
		public static int			totalits;							//Total no. its., no. segment iterations
		public static int			kindex;
		public static double		eta1, eta2, eta3;						//Learning rates
		public static double		sig2;									//Sigma squared
		public static double        sqerror;
		public static double		minerror;								//Sum-squared error of training
		public static double        maxD;									// maximum difference magnitude of all vectors
		
		public static int		    TotalVectors;
		public static int			TotalVectorsInClassification;
			
		public static double[]		fuzzytruth      = new double[2000];
		public static double[]      ClassCenterFuzzyValue = new double[2000];
			
		public static double []		TrainingBelief  = new double[2000];
		public static double []     ClassificationBelief = new double [250000];
		public static double[]      b				= new double[10];			//Biase added at output nodes
		public static double[]      Error           = new double[2000];
		
		public static double[]      xmaxstd			= new double[500];			//Max values for each feature (standardized)
		public static double[]      xminstd			= new double[500];			//Min values for each feature (standardized)
		public static double[,]     TrainingVectors = new double[1000,200];
		public static double[,]     raw 			= new double[50,250000];
		public static double[,]		w				= new double[200,2000];		//RBFLN centers (w's), wts. (u's) on hidden-output lines
		public static double[,]     u				= new double[2000,100];		//RBFLN centers (w's), wts. (u's) on hidden-output lines
		public static double[,]     v				= new double[200,100];		//NN weights from input to output nodes
		public static double[,]		ex				= new double[200,2000];		//Exemplar inputs, target (desired) outputs
		public static double[,]     t				= new double[10,2000];		//Exemplar inputs, target (desired) outputs
		public static double[,]     tt				= new double[2000,10];
		public static double[,]     Classit         = new double[100,2000];
		public static double[,]		y				= new double[2000,2000];		//Outputs of hidden, output nodes
		public static double[,]     z				= new double[50,250000];		//Outputs of hidden, output nodes
		public static double[,]		ClassCenter		= new double[2000,2000]; 
				
		public RbflnFunctions()
		{
			//
			// TODO: Add constructor logic here
			//
		}


		//--------------File Reading Function---------------------		
		
		public static int  readfile1()
		{
	
			int    m,i, n,nn,j,q,qq,k;
			double  xmax, xmin, denom1;

			

			string intText1;

			//----Read Input Exemplar and Output Target Data--------------
		
			if (TrainRBFLN.inFile1 == null)
			{ 
				return 0;
			}	
			
			else

			{ 
				FileStream inStream = new FileStream(TrainRBFLN.inFile1,FileMode.Open, 
					FileAccess.Read,FileShare.None);
				//define a variable for reading stream to
				StreamReader reader = new StreamReader(inStream, Encoding.ASCII);

				//start at beginning of input file again
				inStream.Seek(0,SeekOrigin.Begin);
				
				//read single string value: each is on a different file line
				//convert each string into an integer
																
				intText1 = reader.ReadLine();
				N = int.Parse(intText1);	

				intText1 = reader.ReadLine();
				M = int.Parse(intText1);
				
				intText1 = reader.ReadLine();
				J = int.Parse(intText1);
				
				intText1 = reader.ReadLine();
				Q = int.Parse(intText1);
							
				string [] strArray;			
				char[] charArray = new char[] {','};
								
				for (q = 0; q < Q; q++)
				{
					intText1 = reader.ReadLine();
					strArray = intText1.Split(charArray);
									
					//---------------------begin to read data--------------								
										
					nn=0;

					for (m = 3;m<strArray.Length-1;m++)
					{									
						TrainingVectors[q,nn]	= Convert.ToDouble(strArray[m]);

						nn++;					
					}

					fuzzytruth[q]=Convert.ToDouble(strArray[strArray.Length-1]);

					for (j=0;j<J;j++)
					{
						tt[q,j]=fuzzytruth[q];						
					}
								
					
				} // end of "for q = 0;q<Q " loop


				
				
				//-----------------------define No of classes according to the examplar vector---------
				
				double tmax =  0.0;
				double tmin = 9999.99;
				double threshold = 0.55;
				
			

				for (q=0;q<Q;q++)
				{
					for (j=0;j<J;j++)
					{
						if (tt[q,j] > tmax) tmax = tt[q,j];
						if (tt[q,j] < tmin) tmin = tt[q,j];
					}					
				
				}								
				
				
				for(q=0;q<Q;q++)
				{
					for(n = 0; n<N; n++)
					{
						ex[n,q] = TrainingVectors[q,n];
					}
					
					for(j = 0; j<J; j++)
					{
						//newtt[q,j] = tt[q,j]-tmin;
						//t[j,q] = newtt[q,j];
						t[j,q] = tt[q,j]-tmin;
					}
				}			
				
				//----------------define how many examplar vectors in each class-----------------
				
				i = 0;
				TotalVectors = 0;

				for (q=0;q<Q;q++)
				{ 	    
					for (j=0;j<J;j++)
					{ 		
						if (fuzzytruth[q] > threshold)
						{
							for (n=0;n<N;n++)
							{
								ClassCenter[i,q]=ex[n,q];		//set the value for the vector in cluster
							
								ClassCenterFuzzyValue[i] = fuzzytruth[q];								
							}
							i++;
							TotalVectors++;
													
						}//if (fuzzytruth[q] > threshold)
					}//for (j=0;j<J;j++)
				}// end for for (q=0;q<Q;q++)
				
				/*			
				for (k=0;k<K;k++)
				{
					TotalVectors[k]=0;
				}
  
				for (k=0;k<K;k++)
				{ 
					i=0;	
					for (q=0;q<Q;q++)
					{ 	    
						for (j=0;j<J;j++)
						{ 		
							//if (t[j,q] == k)
							if(fuzzytruth[q] > threshold)
							{
								vect[k,i]=q;				//get the index of new vector															
								i++;			
								TotalVectors[k]++;			// total vectors in each class
							}// end of "if (t[q,j] == k)"	
						} // end of "for (j=0;j<J;j++)"
					}// end of "for (q=0;q<Q;q++)"
					//this.textBox4.AppendText(Convert.ToString(TotalVectorss[k])+ NewLine +Return);
				}// end of "for (k=0;k<K;k++)"
				
				*/
				
				//--------------End Classifying Part---------------------------------	
								
				reader.Close();
				inStream.Close();			
			
				Mref = (int)(5.0*N + 5.0*(Math.Log10(Q)/Math.Log10(2)));
			
				//----standardize each input component------------------------
				//----first, find max and min over all -----------------------
				//----features in all exemplar vectors------------------------
				xmax = ex[0,0]; xmin = xmax;

				for (nn = 1; nn < N;nn++)
				{ 
					for (qq = 1;qq < Q;qq++)
					{ 
						if (ex[nn,qq] >= xmax) xmax = ex[nn,qq];
						if (ex[nn,qq] <= xmin) xmin = ex[nn,qq];
					}
				}
		
				//----if max and min outside of [0,1], standardize------------
				if ((xmax > 1.00) || (xmin < 0.0))
				{
					for (nn=0;nn<N;nn++)
					{ 
						xmax = ex[nn,0]; xmin = xmax;
						for (qq=1;qq<Q;qq++)
						{							
							if (xmax < ex[nn,qq]) xmax = ex[nn,qq];
							if (xmin > ex[nn,qq]) xmin = ex[nn,qq];
						}  //end of loop to standardize each vector input component
					
						denom1 = xmax - xmin; xmaxstd[nn] = xmax; xminstd[nn] = xmin;
					
						for (qq=0;qq<Q;qq++)
						{
							if (denom1 == 0.0) 
							{
								if (xmax == 0.0) 
								{
									ex[nn,qq] = 0.0; //standardize to [0,1]
								} 
								
								else 
								{
									ex[nn,qq] = 1.0; //standardize to [0,1]
								}
						
							} 
							
							else
							{
								ex[nn,qq] = (ex[nn,qq]-xmin)/denom1; //standardize to [0,1]
							}
						}
					}
				}
			} //end of loop to standardize each of N components of input vectors

			return 1;
		}
		//-------------------------end readfile1-------------------------------------------------
		//------------------------read weights from training result file-----------------------
		public static int readtrainingresults()
		{			
			
			string intText1;

			int    m,n, j, q,p;

			double[,]  TempW	= new double[2000,200];
			//double[,]  TempU	= new double[2000,2000];		//RBFLN centers (w's), wts. (u's) on hidden-output lines
		    double[,]  TempV	= new double[2000,50];		//NN weights from input to output nodes

			if (RBFLNClassification.inFile1 == null)
			{ 
				return 0;
			}
			else
			{
				FileStream inStream = new FileStream(RBFLNClassification.inFile1,FileMode.Open, 
					FileAccess.Read,FileShare.None);
				//define a variable for reading stream to
				StreamReader reader = new StreamReader(inStream, Encoding.ASCII);

				//start at beginning of input file again
				inStream.Seek(0,SeekOrigin.Begin);
				
				//read single string value: each is on a different file line
				//convert each string into an integer
																
				intText1 = reader.ReadLine();
				N = int.Parse(intText1);	

				intText1 = reader.ReadLine();
				M = int.Parse(intText1);	

				intText1 = reader.ReadLine();
				J = int.Parse(intText1);

				intText1 = reader.ReadLine();
				TotalVectorsInClassification = int.Parse(intText1);

				intText1 = reader.ReadLine();
				Q = int.Parse(intText1);intText1 = reader.ReadLine();
				sig2 = double.Parse(intText1);

				string [] strArray;			
				char[] charArray = new char[] {'	'};

				for (m = 0; m < M; m++)
				{
					intText1 = reader.ReadLine();
					strArray = intText1.Split(charArray);
									
					//---------------------begin to read data--------------
					//---------------------download w[n,m] from training result----------------			
					n=0;

					for (q = 0;q<strArray.Length-1;q++)
					{									
						//temp[m,q] = Convert.ToDouble(strArray[q]);	
						
						TempW[m,n] = Convert.ToDouble(strArray[q]);
					   
						//x[m,n] = temp[m,q];
										
						n++;					
					}					
				} 

				for(m=0;m<M;m++)
				{
					for(n = 0; n<N; n++)
					{
						w[n,m] = TempW[m,n];
					}
				}
				//---------------------the end of downloading w[n,m]---------------------
				//---------------------download u[m,j] from training result----------------
				for (m = 0; m < M; m++)
				{
					intText1 = reader.ReadLine();
					strArray = intText1.Split(charArray);
					//-----------demonstrate the input data----------------
					//this.textBox4.AppendText(intText1+ NewLine + Return);	
					//---------------------begin to read data--------------
					j=0;

					for (q = 0;q<strArray.Length-1;q++)
					{									
						//TempU[m,q] = Convert.ToDouble(strArray[q]);	
						
						u[m,j] = Convert.ToDouble(strArray[q]);
					   
						//u[m,j] = TempU[m,q];
										
						j++;					
					}			
					
				}
				//---------------------the end of downloading u[m,j]---------------------
				//---------------------download v[n,j] from training result----------------

				for (j = 0; j < J; j++)
				{
					intText1 = reader.ReadLine();
					strArray = intText1.Split(charArray);
					//-----------demonstrate the input data----------------
					//this.textBox4.AppendText(intText1+ NewLine + Return);	
					//---------------------begin to read data--------------
								
					n=0;

					for (q = 0;q<strArray.Length-1;q++)
					{									
						//temp[j,q] = Convert.ToDouble(strArray[q]);							
					   
						TempV[j,n] = Convert.ToDouble(strArray[q]);	
										
						n++;					
					}					
				} 
				for(j=0;j<J;j++)
				{
					for(n = 0; n<N; n++)
					{
						v[n,j] = TempV[j,n];
					}
				}
				
				//-------------------read b[j]----------------

				for (j = 0; j < J; j++)
				{				
					intText1 = reader.ReadLine();
					b[j] = double.Parse(intText1);
			
					//this.textBox4.AppendText(Convert.ToString(b[j]) + NewLine + Return);
				
				}
				//---------------Read TotalVectors from training result---------
				
								
					//intText1 = reader.ReadLine();
					//TotalVectors = int.Parse(intText1);	
				

				// the end of reading TotalVectors----------------------
				/*
				
				// -----------------Read vect[k,i] from training result--------
				//-------------read vect[k,i]----------------------
			
				
					intText1 = reader.ReadLine();
					strArray = intText1.Split(charArray);
												
					i=0;

					for (q = 0;q<strArray.Length-1;q++)
					{									
						//temp1[k,q] = Convert.ToInt32(strArray[q]);							
					   
						//vect[k,i] = Convert.ToInt32(strArray[q]);	

						//vect[k,i] = temp1[k,q];
										
						i++;					
					}			
					
			
				// --------the end of reading vect[k,i]----------------
				
				*/
				
				
				
				
				//---------read t[j,q] from training result------------
				for (q = 0; q < Q; q++)
				{
					intText1 = reader.ReadLine();
					strArray = intText1.Split(charArray);
					//-----------demonstrate the input data----------------
					//this.textBox4.AppendText(intText1+ NewLine + Return);	
				
					//---------------------begin to read data--------------
								
					j=0;

					for (p = 0;p<strArray.Length-1;p++)
					{									
						tt[q,j] = Convert.ToDouble(strArray[p]);							
																		
						j++;					
					}	
				}

				for(q=0;q<Q;q++)
				{				
					
					for(j = 0; j<J; j++)
					{
						Classit[j,q] = tt[q,j];
					}
				}
			}
			return 1;
		}
		//------------------------the end of weights reading------------------------
		//------------------------Read raw data file------------------------------
		public static int readfile2()
		{
			string intText2;
			int    m,nn,rr;
			double  xmax2, xmin2, denom2;
			
			double[]      xmaxstd2			= new double[100];			//Max values for each feature (standardized)
			double[]      xminstd2		 	= new double[100];
						

			//----Read Input Exemplar and Output Target Data--------------
		
			if (RBFLNClassification.inFile2 == null)
			{ 
				return 0;
			}	
			
			else

			{ 
				FileStream inStream = new FileStream(RBFLNClassification.inFile2,FileMode.Open, 
					FileAccess.Read,FileShare.None);
				//define a variable for reading stream to
				StreamReader reader = new StreamReader(inStream, Encoding.ASCII);

				//start at beginning of input file again
				inStream.Seek(0,SeekOrigin.Begin);
				
				//read single string value: each is on a different file line
				//convert each string into an integer
																
				intText2 = reader.ReadLine();
				N = int.Parse(intText2);
				
				intText2 = reader.ReadLine();
				M1 = int.Parse(intText2);
		
				intText2 = reader.ReadLine();
				J = int.Parse(intText2);
				
				intText2 = reader.ReadLine();
				R = int.Parse(intText2);
							
				string [] strArray;			
				char[] charArray = new char[] {','};				
				
				for (rr = 0; rr < R; rr++)
				{
					intText2 = reader.ReadLine();
					strArray = intText2.Split(charArray);
					//-----------demonstrate the input data----------------
					//this.textBox4.AppendText(intText1+ NewLine + Return);	
				
					//---------------------begin to read data--------------
								
					nn=0;

					for (m = 3;m<strArray.Length-1;m++)
					{									
						//temp[rr,m] = Convert.ToDouble(strArray[m]);							
					   
						raw[nn,rr] = Convert.ToDouble(strArray[m]);	
										
						nn++;					
					}			
					
				}


				for(rr=0;rr<R;rr++)
				{
					for(nn = 0; nn<N; nn++)
					{
						if ((raw[nn,rr]==0)&(raw[nn+1,rr]==0))

						{
							raw[nn,rr] = 0.00001;
						}

					}
				}
				
				reader.Close();
				inStream.Close();			
			
				//----standardize each input component------------------------
				//----first, find max and min over all -----------------------
				//----features in all exemplar vectors------------------------
				xmax2 = raw[0,0]; xmin2 = xmax2;

				for (nn = 1; nn < N;nn++)
				{ 
					for (rr = 1;rr < R;rr++)
					{ 
						if (raw[nn,rr] >= xmax2) xmax2 = raw[nn,rr];
						if (raw[nn,rr] <= xmin2) xmin2 = raw[nn,rr];
					}
				}
		
				//----if max and min outside of [0,1], standardize------------
				if ((xmax2 > 1.00) || (xmin2 < 0.0))
				{
					stdFlag = 1;

					for (nn=0;nn<N;nn++)
					{ 
						xmaxstd2[nn] = xmax2;
						xminstd2[nn] = xmin2;
						denom2 = xmaxstd2[nn]-xminstd2[nn];
					
						for (rr=0;rr<R;rr++)
						{
							if (denom2 == 0.0) 
							{
								if (xmaxstd2[nn] == 0.0) 
								{
									raw[nn,rr] = 0.0; //standardize to [0,1]
								} 
								
								else 
								{
									raw[nn,rr] = 1.0; //standardize to [0,1]
								}
						
							} 
							
							else
							{
								raw[nn,rr] = (raw[nn,rr]-xminstd2[nn])/denom2; //standardize to [0,1]
							}
						}
					}
					
				}

				else stdFlag = 0;
				
			} //end of loop to standardize each of N components of input vectors
			
			return 1;
		}
		//----end readfile2------------------------------------------------
		
		//----Learnmenu--------------called by Main()----------------------
		//---------------------------sets up training situation------------
		public static void learnmenu()		
		{			
			int    n, m, q;
			//----Set Initial Learning Rates------------------------------
			eta1 = 6.0; eta2 = 6.0; eta3 = 6.0;
			//------Get Sigma-Squared or Default Value--------------------
			sig2 = 1.0/(4.0*Math.Pow(RbflnM.M,(1.0/N)));
			sig2 = sig2*sig2;

			drawweights();  

			totalits = 0;

			if (RbflnM.M <= Q)
			{
				for (m=0;m<RbflnM.M;m++)
				{
					for (n=0;n<N;n++) w[n,m] = ex[n,m];
				}
			}
			else
			{
				for (q=0;q<Q;q++)
				{
					for (n=0;n<N;n++) w[n,q] = ex[n,q];
				}
				drawcenters();
			}
			learn2();
			return;
		}
		//----end learnmenu-----------------------------------------------
		//----Learn2---------------called by Learnmenu()------------------
		//-------------------------adjusts weights-training---------------
		public static void learn2()
		{		  
			evalnet();
			double	temperror;
			double  sqerror;
			double  delu, delv, delb;
			int		i, n, m, j, q , MpN, BigIts = 5;
			int		bigits, localits, Uits = 2, Vits = 2, Bits = 1;
			localits = 0; 
			MpN = RbflnM.M + N;           
			minerror = evalnet();

			for (j =0; j<J; j++)
			{
				b[j] = 0.1;
			}

			lbl:
				for (bigits=0;bigits<BigIts;bigits++)
				{ 
					//----adjust weights u[m][j] at output nodes----------------
					for (i=0;i<Uits;i++)   //adjust hidden-output node weights twice
					{
						for (j=0;j<J;j++)    //Uits = 2  
						{
							for (m=0;m<RbflnM.M;m++)
							{
								delu = 0.0;
								for (q=0;q<Q;q++) delu = delu + (t[j,q]-z[j,q])*y[m,q];
								u[m,j] = u[m,j] + eta1*delu/MpN;
							}
						}
						//----en route learning rate for hidden-output weights------
						temperror = evalnet();  //En Route adjustment of learning rate
						if (temperror < minerror) eta1 = 1.028*eta1;
						else                      eta1 = 0.952*eta1;
						minerror = temperror;
					}
					//----adjust weights v[n][j] at output nodes------------------
					for (i=0;i<Vits;i++)    //adjust input-output node weights twice here
					{
						for (j=0;j<J;j++)     //Vits = 2
						{
							for (n=0;n<N;n++)
							{
								delv = 0.0;
								for (q=0;q<Q;q++) delv = delv + (t[j,q]-z[j,q])*ex[n,q];
								v[n,j] = v[n,j] + eta2*delv/MpN;
							}
						}
						//----en route learning rate for input/output weights-------
						temperror = evalnet();  //En Route adjustment of learning rate
						if (temperror < minerror) eta2 = 1.028*eta2;
						else                      eta2 = 0.952*eta2;
						minerror = temperror;
					}
					//----adjust biases b[j] at output nodes----------------------
					for (i=0;i<Bits;i++)      //adjust output biases once here
					{
						for (j=0;j<J;j++)       //Bits = 1
						{
							delb = 0.0;
							for (q=0;q<Q;q++) delb = delb + (t[j,q] - z[j,q]);
							
							b[j] = b[j] + eta3*delb/MpN;
							
						}
						temperror = evalnet();  //En Route adjustment of learning rate
						if (temperror < minerror) eta3 = 1.024*eta3;
						else                      eta3 = 0.960*eta3;
						minerror = temperror;
					}
				}
			//----end for loop on bigits------------------------------------
			localits = localits + BigIts*2;
			totalits = totalits + BigIts*2;
			//----print sum-squared error-----------------------------------
			sqerror = minerror;
			
			//----------------------------------------------------------------
			if (localits < RbflnM.its) goto lbl;
			return;
		}
		//----end learn2--------------------------------------------------
		//----Evalnet-------------called by Learn2() / Error()------------
		//------------------------evaluates all NN values-----------------
		public static double evalnet()
		{		
			int    n, m, j, q, MpN;
			double d, sx, sy, val, rvalue;
			 
			rvalue = 0;
	 

			MpN = RbflnM.M + N;
			//----Compute outputs y[q,m] at hidden nodes-------------------
			for (m=0;m<RbflnM.M;m++)
			{
				for (q=0;q<Q;q++)
				{
					sx = 0.0;
					for (n=0;n<N;n++)
					{
						d = ex[n,q] - w[n,m]; 
						sx = sx + d*d;
					}
					y[m,q] = Math.Exp(-sx/(2.0*sig2));
				}
			}
			for (q=0;q<Q;q++)
			{
				for (j=0;j<J;j++)
				{
					sx = 0.0; sy = 0.0;
					for (m=0;m<RbflnM.M;m++) sy = sy + y[m,q]*u[m,j];
					for (n=0;n<N;n++) sx = sx + ex[n,q]*v[n,j];
					z[j,q] = (sx + sy + b[j])/MpN;
				}
			}
			
	/*-------Calculate training belief----------------------*/
			
			for (j=0;j<J;j++)
			{
				for (q=0;q<Q;q++)
				{
					TrainingBelief [q]  = Math.Abs (z[j,q]);
				}
			}

			
			for (j=0;j<J;j++)
			{
				for (q=0;q<Q;q++)
				{
					val = z[j,q] - t[j,q];
					//Error [q] = Math.Abs(val);
					rvalue = rvalue + val*val;
				}
			}
			rvalue = rvalue/Q;	
				
			return rvalue;
		}
		//----end evalnet-------------------------------------------------
		
		//----------------------------------------------------------------

		public static void drawweights()
		{
			int   j, m, n;
			double RanDivisor;
			RanDivisor = 32767.0*2.0;

			Random R = new Random (DateTime.Now.Millisecond);
			//----draw weights between -0.5 and 0.5-------------------------
			for (j=0;j<J;j++)
			{
				for (n=0;n<N;n++) v[n,j] = R.Next(33768)/(RanDivisor - 0.25);
				for (m=0;m<RbflnM.M;m++) u[m,j] = R.Next(33768)/(RanDivisor - 0.25);
				b[j] = 0.1;
			}
			return;
		}
		//----end drawweights---------------------------------------------
		//----Drawcenters-----------called by Learnmenu()-----------------
		public static void drawcenters()	
		{
			int     g,n,m,q,qq;
			//double	dist,Dist,sum;
			//double  RanDivisor = 32767.0;
			//double  avedistance = 0.9;

			/*

			if (TrainRBFLN.checkstatus == 0)
			{
				Random R = new Random (DateTime.Now.Millisecond);

				for (n=0;n<N;n++)
				{
					for (m=Q;m<RbflnM.M;m++) w[n,m] = R.Next(33768)/RanDivisor;
				}// end of "for (n=0;n<N;n++)"
			
			}// end of "if (TrainRBFLN.checkstatus == 0)" 
			
			*/

			//if (TrainRBFLN.checkstatus == 1)
			//{								
				/*****************************************************************/
				//-----Find the aveage distance between each vector in Cluster1 - --
				/*
				sum=0;

				for (q=0;q<Q-1;q++)
				{
					for (qq=q+1;qq<Q;qq++)
					{	
						dist = 0;
						for (n=0;n<N;n++)
						{	
							dist = dist + Math.Pow((ClassCenter[q,n] - ClassCenter[qq,n]),2);
						}
						sum = sum +Math.Sqrt(dist);
					}	  	
				}	
				*/				
				
				//-----------------End of Average Distance Calculation----------------------
				//***************************************************************************
				//- - - - - - - -Eliminate Centers too Close to Others - - - ----------------
				
			
			
			/*-------------------Find distance between pairs---------------------------*/
								
				
			/*
			
			for (q=0;q<TotalVectors-1;q++)     //For each unique pair of centers
				{          
					for (qq=q+1;qq<TotalVectors;qq++)
					{
						dist = 0.0;
						for (n=0;n<N;n++) 
						{
							dist = dist + Math.Pow((ClassCenter[q,n] - ClassCenter[qq,n]),2);
						} // end of "for (n=0;n<N;n++)"
					
						Dist = Math.Sqrt(dist);	
			
						if (Dist < avedistance)
						{
							for (g=qq; g<TotalVectors-1; g++)
							{
								if(ClassCenterFuzzyValue[g]<ClassCenterFuzzyValue[g+1])
								{
									for (n=0;n<N;n++) ClassCenter[g,n] = ClassCenter[g+1,n];  //eliminate one vector
										
								}// end of "if(fuzzyValue[g]<fuzzyValue[g+1])"

								else
								{
									for (n=0;n<N;n++) ClassCenter[g+1,n] = ClassCenter[g,n];  //eliminate one vector
								}// end of " else"										
										                          
							}// end of "for (g=qq; g<Q-1; g++)"

							TotalVectors--;								
							
						}// end of "if (dist < avedistance)"
													
					}// end of "for (qq=q+1;qq<Q;qq++)"			
				}// end of "for (q=0;q<Q-1;q++)"			
			
			
			//}// end of "if (TrainRBFLN.checkstatus == 1)"

			*/
			

			for(q=0;q<TotalVectors;q++)
			{
				for(n = 0; n<N; n++)
				{
					w[n,q] = ClassCenter[q,n];
				}
			}			

			return;
		}

		//----end drawcenters---------------------------------------------
		//----Error-----------------called by Main()----------------------

		public static void error()
		{			
			sqerror = evalnet();
		
			return;
		}
		//----end error---------------------------------------------------
		public static void execute()
		{
			
			//----chose feature vectors from keyboard or test file---------
			
			//----FILE input is selected for feature vector source---------
		
			autoruns();
			
			return;		
		}
		//----Xevalnet-----------called by Execute() / Autoruns()---------
		public static double xevalnet(int rr, int qq, double[] xxi)
		{ 
			int    n, m, r,j, MpN;
			double ry, sx, sy, val2, rvalue, diff;
			double[]        yz = new double [1000];

			rvalue = 0.0; MpN = M + N;

			for (m=0;m<M;m++)
			{
				ry = 0.0;
				for (n=0;n<N;n++)
				{
					diff = xxi[n] - w[n,m]; ry = ry + diff*diff;
				}
				//yy[m,rr] = Math.Exp(-ry/(2.0*sig2));
				yz[m]= Math.Exp(-ry/(2.0*sig2));
			}
			
			for (j=0;j<J;j++)
			{
				sx = 0.0; sy = 0.0;
				
				//for (m=0;m<M;m++) sy = sy + yy[m,rr]*u[m,j];

				for (m=0;m<M;m++) sy = sy + yz[m]*u[m,j];
				
				for (n=0;n<N;n++) sx = sx + xxi[n]*v[n,j];
				
				z[j,rr] = (sx + sy + b[j])/MpN;
				
			}

			for (j=0;j<J;j++)
			{
				for (r=0;r<R;r++)
				{
					ClassificationBelief [r]  = Math.Abs (z[j,r]);
				}
			}

			for (j=0;j<J;j++)
			{
				val2  = z[j,rr] - Classit[j,qq];				
				rvalue = rvalue+val2*val2;
			}

			return rvalue;
		}
		//----end xevalnet-------------------------------------------------

		//----Autoruns----------called by Main() / Execute()---------------
		//----------------------puts each vector through NN, gets class----
		public static void autoruns()
		{
			int			k,i,j,n,r, q, decision;
			double		miniz;
			
			
			double[]	er = new double[1000];
			double[]    xx = new double[200];			
			
			for (r=0;r<R;r++)                       //Test NN on each feature vector
			{
				//totalValue[r]=0;
				
				//for (k=0;k<K;k++)
				//{
					//FitCluster[r,k]=0;
				//}
				
				
				for (n=0;n<N;n++)   xx[n] = raw[n,r]; //Build a feature vector
				
				for (q=0;q<Q;q++) er[q] = xevalnet(r,q,xx); //Classify vector
				
				miniz = (float) 99999.9;  decision = 0;

				/*
				//-------find class with minimum error------------
				for (q=0;q<Q;q++)
				{
					if (er[q] < miniz)
					{
						miniz = er[q]; decision = q;
					}					
				}

				Classif[r] = q;
				
				for ( k = 0; k<K; k++)
				{
					for ( i = 0 ; i< TotalVectors[k]; i++)
					{
						if (decision-vect[k,i]==0)
						{				
							Classif[r] = k;
							kindex = k;
						}
					}				
				}
			
				
				for (j=0;j<J;j++)
				{
					//FitCluster[r,kindex] = 1.0-(Math.Abs((z[j,r]-kindex)/(z[j,r]+1)));
					FitCluster[r,kindex] =(Math.Abs((z[j,r]-kindex)/(z[j,r]+1)));

				}
				
				Goodness[r] = FitCluster[r,kindex];	
				
					*/							

			}
			
			return;
		}
		//----end autoruns------------------------------------------------
		
		
		
		


	
		
				
		//**************************End of RbflnFunctions**********************

	}
}
